#! /usr/bin/python
# --*-- encoding:utf-8 --*--
'''
Parse and get radio list from baidu. play radio with gstreamer.
author:Shrek Zhou(zgwmike@hotmail.com)
'''

from urllib2 import urlopen
from urllib import quote,unquote
from lxml import etree
import pygst
pygst.require("0.10")
import gst
import gobject
import sys
import os

BBC_LIVE='http://www.bbc.co.uk/worldservice/meta/tx/nb/live_infent_au_nb.asx'
def verifyPkgs():
	pkgs='gstreamer0.10-plugins-bad gstreamer0.10-plugins-ugly gstreamer0.10-plugins-good python-gst python-lxml'
	import apt
	c = apt.cache.Cache(apt.progress.OpTextProgress())
	pkgs=pkgs.split(' ')
	for pkg in pkgs:
		if c.has_key(pkg):
			p=c[pkg]
			print '======================='
			print 'installing %s'%c[pkg].name
			if not p.isInstalled:
				p.markInstall()
			else:
				print '%s is already installed!'%pkg
		else:
			print '%s does not exists!'%pkg
		c.commit(apt.progress.TextFetchProgress(), apt.progress.InstallProgress())
def readurl(url,encoding=None):
    try:
		con=urlopen(url)
		resp=con.read()
		con.close()
		if encoding is not None:
			return etree.HTML(resp.decode('gb18030'))
		else:
			return etree.HTML(resp)
    except UnicodeDecodeError,e:
		logger.exception('error decode html page %s',url)
		return etree.HTML(resp)
    except UnicodeEncodeError,e:
		logger.exception('error encode html page %s',url)
		return etree.HTML(resp)
    except:
		logger.exception('error while reading url %s',url)
		return etree.HTML('<html></html>')
        
def printtag(tag):
        print etree.tostring(tag,pretty_print=True)
        
class AsxParser(object):
	def parse(self):
		root=urlopen(self.asxsrc).read()
		root=etree.HTML(root)
		refs=root.xpath(".//ref")
		for ref in refs:
			href=ref.get('href','')
			if href.endswith('.asx'):
				self.asxsrc=href
				self.parse()
			else:
				self.playlist.append(href)
		return self.playlist
	def __init__(self,asxfile):
		if os.path.isfile(asxfile):
			self.asxsrc=open(asxfile).read().strip()
		else:
			self.asxsrc=asxfile
		self.playlist=[]
class BaiduRadio(object):
	rooturl='http://list.mp3.baidu.com/radio/iframe.html'
	M_PLAY='Play'
	M_RECORD='Record'
	def __init__(self):
		self.radios={}
		self.parseBBC()
		self.player=gst.element_factory_make('playbin','player')		
		bus=self.player.get_bus()
		#bus.enable_sync_message_emission()
		bus.add_signal_watch()
		#bus.connect('sync-message::element',self.on_sync_msg)
		bus.connect('message',self.on_message)
	def parseBBC(self):
		try:
			asxparser=AsxParser(BBC_LIVE)
			asxparser.parse()			
			if len(asxparser.playlist)>0:
				self.radios['BBC World Service']=asxparser.playlist[0]
		except Exception,e:
			print e
	def on_message(self,bus,message):
		t = message.type
		print message
		'''
        if t == gst.MESSAGE_ERROR:
            err, debug = message.parse_error()
            print "Error: %s" % err, debug
            if self.on_eos:
                self.on_eos()
            self.playing = False
        elif t == gst.MESSAGE_EOS:
            if self.on_eos:
                self.on_eos()
            self.playing = False
        print message
        '''
	def getradiolist(self):
		root=readurl(self.rooturl,'gb2312')
		for a in root.xpath(".//a"):
			href=a.get('href')
			if not href:continue
			if href.startswith('mms') or href.startswith('rtsp'):
				self.radios[a.text]=a.get('href').strip()
				#print u'电台：%s, 频率：%s'%(a.text,a.get('href'))		
		return self.radios
	def playradio(self,radiourl,mode=M_PLAY,savefile="/home/shrek/radio.ogg"):
		#import pdb;pdb.set_trace()
		self.player.set_property('uri', radiourl)		
		if mode==self.M_RECORD:
			self.player.set_property('audio-sink',RecorderBin(savefile))		
		self.player.set_state(gst.STATE_PLAYING)		
	def stop(self):
		self.player.set_state(gst.STATE_NULL)
		
class RecorderBin(gst.Bin):
    '''record stream while playing....
    
			    -->vorbisenc-->oggmux-->filesink            #save stream to file.
    audioconvert-->mymux-->
			    -->queue-->audioconvert-->alsasink	#push stream to audio card to play it.
    '''
    def __init__(self,output):
		self.__gobject_init__()
		conv=gst.element_factory_make("audioconvert", "converter")
		mymux=gst.element_factory_make("tee")
		self.add(conv,mymux)
		conv.link(mymux)
		
		enc = gst.element_factory_make("vorbisenc", "vorbis-encoder")
		create = gst.element_factory_make("oggmux", "ogg-create")
		fileout = gst.element_factory_make("filesink", "sink")
		fileout.set_property("location", output)
		self.add(enc,create,fileout)		
		gst.element_link_many(enc,create,fileout)	
		
		alsaconv=gst.element_factory_make('audioconvert')
		alsaqueue=gst.element_factory_make('queue')
		alsasink=gst.element_factory_make('alsasink')
		self.add(alsaconv,alsaqueue,alsasink)
		
		mymux.link(enc)
		gst.element_link_many(mymux,alsaqueue,alsaconv,alsasink)
		
		self.add_pad(gst.GhostPad('sink',conv.get_pad('sink')))
	
if __name__=='__main__':
	prompt='''
退出：请按‘q’键
换台：请按‘s‘键
显示电台：请按’p‘键
如果是第一次运行该程序，强烈建议以root用户执行一次， 以便我检查相关依赖，保证收听正常。
'''
	import pwd
	if pwd.getpwuid(os.geteuid())[0]=='root':
		verifyPkgs()
	def play():
		pmp=u'你要收听哪个电台(输入你要收听电台的序号，如8)? \nq，退出; p打印电台列表'
		while True:
			print pmp
			x=raw_input()
			if x=='q':
				sys.exit()
			elif x=='p':
				printradios()
				print pmp
				x=raw_input()
			try:
				x=int(x)
				break
			except:
				pass
		sortedkey=sorted(radios.keys())[x]
		print u'正在收听电台---%s'%sortedkey
		try:
			
			radio.playradio(quote(radios[sortedkey].encode('utf-8'),safe=':/'))
		except Exception,e:
			print e
	def printradios():
		x=0
		rstr=''
		i=0
		for k in sorted(radios):
			if x<1:
				rstr +="%s. %-40s\t"%(i,k)
				x+=1
				i+=1
				#continue
			else:
				x=0
				rstr += "\n%s. %-40s\t"%(i,k)
				i+=1
			#i+=1
		print rstr
	gobject.threads_init()
	radio=BaiduRadio()
	radios=radio.getradiolist()	
	printradios()
	play()
	while True:
		inp=raw_input(prompt)
		if inp.strip()=='q':
			radio.stop()
			sys.exit()
		elif inp.strip()=='s':
			radio.stop()
			play()
		elif inp.strip()=='p':
			printradios()
		
		
